/*
   Copyright 2006-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.util;

import org.apache.log4j.Logger;

/**
 * System clock based unique ID generator.
 *
 * @author  murali
 */
public class UniqueId {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger(UniqueId.class);

    private static UniqueId INSTANCE = new UniqueId();

    /**
     * The digits for a base-36 number.
     */
    private static char[] HASH = {
        '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
        'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm',
        'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z'
    };


    //------------------------------------------------------------
    //- Class Functions

    public static String next() {
        return String.valueOf(INSTANCE.nextHash());
    }


    protected static String toHash(long value) {
        StringBuffer buffer = new StringBuffer();

        if (value == 0) {
            buffer.append('0');
        }

        while (value > 0) {
            int remainder = (int) (value % HASH.length);
            buffer.append(HASH[remainder]);
            value = value / HASH.length;
        }

        return buffer.reverse().toString();
    }


    //------------------------------------------------------------
    //- Instance Variables

    private long fLast = 0;

    //------------------------------------------------------------
    //- Constructors


    //------------------------------------------------------------
    //- Accessors

    protected long getLast() {
        return fLast;
    }


    //------------------------------------------------------------
    //- Settors

    protected void setLast(long pLast) {
        fLast = pLast;
    }


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    private synchronized long nextLong() {
        return WallClock.currentTimeMillis();
    }


    //------------------------------------------------------------
    //- Public Interface Functions

    public synchronized String nextHash() {
        long value = nextLong();
        while (value == getLast()) {
            try {
                Thread.sleep(1);
            } catch (InterruptedException ie) {
                // nothing.
            }
            value = nextLong();
        }
        setLast(value);

        return toHash(value);
    }


    //------------------------------------------------------------
    //- Class Interface Functions



    //------------------------------------------------------------
    //- Inner Classes



    //------------------------------------------------------------
    //- Main

    public static void main(String[] args) {
        try {
            LOG.info(next());
        } catch (Throwable thr) {
            LOG.error("Fatal error.", thr);
        }
    }



}
